@@ -2,14 +2,15 @@
 
 mod boxed_derive;
 mod clone;
+mod clone_old;
 mod closure;
+mod closure_old;
 mod derived_properties_attribute;
 mod downgrade_derive;
 mod enum_derive;
 mod error_domain_derive;
 mod flags_attribute;
-mod object_interface_attribute;
-mod object_subclass_attribute;
+mod object_impl_attributes;
 mod properties;
 mod shared_boxed_derive;
 mod value_delegate_derive;
@@ -18,7 +19,7 @@ mod variant_derive;
 mod utils;
 
 use flags_attribute::AttrInput;
-use proc_macro::TokenStream;
+use proc_macro::{TokenStream, TokenTree};
 use proc_macro2::Span;
 use syn::{parse_macro_input, DeriveInput};
 use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
@@ -73,9 +74,12 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 /// use std::rc::Rc;
 ///
 /// let v = Rc::new(1);
-/// let closure = clone!(@strong v => move |x| {
-///     println!("v: {}, x: {}", v, x);
-/// });
+/// let closure = clone!(
+///     #[strong] v,
+///     move |x| {
+///         println!("v: {}, x: {}", v, x);
+///     },
+/// );
 ///
 /// closure(2);
 /// ```
@@ -88,9 +92,13 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 /// use std::rc::Rc;
 ///
 /// let u = Rc::new(2);
-/// let closure = clone!(@weak u => move |x| {
-///     println!("u: {}, x: {}", u, x);
-/// });
+/// let closure = clone!(
+///     #[weak]
+///     u,
+///     move |x| {
+///         println!("u: {}, x: {}", u, x);
+///     },
+/// );
 ///
 /// closure(3);
 /// ```
@@ -98,7 +106,7 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 /// #### Allowing a nullable weak reference
 ///
 /// In some cases, even if the weak references can't be retrieved, you might want to still have
-/// your closure called. In this case, you need to use `@weak-allow-none`:
+/// your closure called. In this case, you need to use `#[weak_allow_none]` instead of `#[weak]`:
 ///
 /// ```
 /// use glib;
@@ -109,11 +117,15 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 ///     // This `Rc` won't be available in the closure because it's dropped at the end of the
 ///     // current block
 ///     let u = Rc::new(2);
-///     clone!(@weak-allow-none u => @default-return false, move |x| {
-///         // We need to use a Debug print for `u` because it'll be an `Option`.
-///         println!("u: {:?}, x: {}", u, x);
-///         true
-///     })
+///     clone!(
+///         #[weak_allow_none]
+///         u,
+///         move |x| {
+///             // We need to use a Debug print for `u` because it'll be an `Option`.
+///             println!("u: {:?}, x: {}", u, x);
+///             true
+///         },
+///     )
 /// };
 ///
 /// assert_eq!(closure(3), true);
@@ -126,10 +138,13 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 /// use glib_macros::clone;
 ///
 /// let v = "123";
-/// let closure = clone!(@to-owned v => move |x| {
-///     // v is passed as `String` here
-///     println!("v: {}, x: {}", v, x);
-/// });
+/// let closure = clone!(
+///     #[to_owned] v,
+///     move |x| {
+///         // v is passed as `String` here
+///         println!("v: {}, x: {}", v, x);
+///     },
+/// );
 ///
 /// closure(2);
 /// ```
@@ -143,18 +158,28 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 ///
 /// let v = Rc::new(1);
 /// let u = Rc::new(2);
-/// let closure = clone!(@strong v as y, @weak u => move |x| {
-///     println!("v as y: {}, u: {}, x: {}", y, u, x);
-/// });
+/// let closure = clone!(
+///     #[strong(rename_to = y)]
+///     v,
+///     #[weak] u,
+///     move |x| {
+///         println!("v as y: {}, u: {}, x: {}", y, u, x);
+///     },
+/// );
 ///
 /// closure(3);
 /// ```
 ///
-/// ### Providing a default return value if upgrading a weak reference fails
+/// ### Providing a return value if upgrading a weak reference fails
 ///
-/// You can do it in two different ways:
+/// By default, `()` is returned if upgrading a weak reference fails. This behaviour can be
+/// adjusted in two different ways:
 ///
-/// Either by providing the value yourself using `@default-return`:
+/// Either by providing the value yourself using one of
+///
+///   * `#[upgrade_or]`: Requires an expression that returns a `Copy` value of the expected return type,
+///   * `#[upgrade_or_else]`: Requires a closure that returns a value of the expected return type,
+///   * `#[upgrade_or_default]`: Requires that the return type implements `Default` and returns that.
 ///
 /// ```
 /// use glib;
@@ -162,10 +187,15 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 /// use std::rc::Rc;
 ///
 /// let v = Rc::new(1);
-/// let closure = clone!(@weak v => @default-return false, move |x| {
-///     println!("v: {}, x: {}", v, x);
-///     true
-/// });
+/// let closure = clone!(
+///     #[weak] v,
+///     #[upgrade_or]
+///     false,
+///     move |x| {
+///         println!("v: {}, x: {}", v, x);
+///         true
+///     },
+/// );
 ///
 /// // Drop value so that the weak reference can't be upgraded.
 /// drop(v);
@@ -173,17 +203,21 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 /// assert_eq!(closure(2), false);
 /// ```
 ///
-/// Or by using `@default-panic` (if the value fails to get upgraded, it'll panic):
+/// Or by using `#[upgrade_or_panic]`: If the value fails to get upgraded, it'll panic.
 ///
 /// ```should_panic
 /// # use glib;
 /// # use glib_macros::clone;
 /// # use std::rc::Rc;
 /// # let v = Rc::new(1);
-/// let closure = clone!(@weak v => @default-panic, move |x| {
-///     println!("v: {}, x: {}", v, x);
-///     true
-/// });
+/// let closure = clone!(
+///     #[weak] v,
+///     #[upgrade_or_panic]
+///     move |x| {
+///         println!("v: {}, x: {}", v, x);
+///         true
+///     },
+/// );
 /// # drop(v);
 /// # assert_eq!(closure(2), false);
 /// ```
@@ -192,7 +226,7 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 ///
 /// Here is a list of errors you might encounter:
 ///
-/// **Missing `@weak` or `@strong`**:
+/// **Missing `#[weak]` or `#[strong]`**:
 ///
 /// ```compile_fail
 /// # use glib;
@@ -200,7 +234,10 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 /// # use std::rc::Rc;
 /// let v = Rc::new(1);
 ///
-/// let closure = clone!(v => move |x| println!("v: {}, x: {}", v, x));
+/// let closure = clone!(
+///     v,
+///     move |x| println!("v: {}, x: {}", v, x),
+/// );
 /// # drop(v);
 /// # closure(2);
 /// ```
@@ -216,9 +253,12 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 ///
 /// impl Foo {
 ///     fn foo(&self) {
-///         let closure = clone!(@strong self => move |x| {
-///             println!("self: {:?}", self);
-///         });
+///         let closure = clone!(
+///             #[strong] self,
+///             move |x| {
+///                 println!("self: {:?}", self);
+///             },
+///         );
 ///         # closure(2);
 ///     }
 /// }
@@ -235,9 +275,13 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 ///
 /// impl Foo {
 ///     fn foo(&self) {
-///         let closure = clone!(@strong self as this => move |x| {
-///             println!("self: {:?}", this);
-///         });
+///         let closure = clone!(
+///             #[strong(rename_to = this)]
+///             self,
+///             move |x| {
+///                 println!("self: {:?}", this);
+///             },
+///         );
 ///         # closure(2);
 ///     }
 /// }
@@ -256,9 +300,12 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 ///
 /// impl Foo {
 ///     fn foo(&self) {
-///         let closure = clone!(@strong self.v => move |x| {
-///             println!("self.v: {:?}", v);
-///         });
+///         let closure = clone!(
+///             #[strong] self.v,
+///             move |x| {
+///                 println!("self.v: {:?}", v);
+///             },
+///         );
 ///         # closure(2);
 ///     }
 /// }
@@ -275,16 +322,31 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 /// # }
 /// impl Foo {
 ///     fn foo(&self) {
-///         let closure = clone!(@strong self.v as v => move |x| {
-///             println!("self.v: {}", v);
-///         });
+///         let closure = clone!(
+///             #[strong(rename_to = v)]
+///             self.v,
+///             move |x| {
+///                 println!("self.v: {}", v);
+///             },
+///         );
 ///         # closure(2);
 ///     }
 /// }
 /// ```
 #[proc_macro]
 pub fn clone(item: TokenStream) -> TokenStream {
-    clone::clone_inner(item)
+    // Check if this is an old-style clone macro invocation.
+    // These always start with an '@' punctuation.
+    let Some(first) = item.clone().into_iter().next() else {
+        return syn::Error::new(Span::call_site(), "expected a closure or async block")
+            .to_compile_error()
+            .into();
+    };
+
+    match first {
+        TokenTree::Punct(ref p) if p.to_string() == "@" => clone_old::clone_inner(item),
+        _ => clone::clone_inner(item),
+    }
 }
 
 /// Macro for creating a [`Closure`] object. This is a wrapper around [`Closure::new`] that
@@ -300,16 +362,20 @@ pub fn clone(item: TokenStream) -> TokenStream {
 ///
 /// Similarly to [`clone!`](crate::clone!), this macro can be useful in combination with signal
 /// handlers to reduce boilerplate when passing references. Unique to `Closure` objects is the
-/// ability to watch an object using a the `@watch` directive. Only an [`Object`] value can be
-/// passed to `@watch`, and only one object can be watched per closure. When an object is watched,
+/// ability to watch an object using the `#[watch]` attribute. Only an [`Object`] value can be
+/// passed to `#[watch]`, and only one object can be watched per closure. When an object is watched,
 /// a weak reference to the object is held in the closure. When the object is destroyed, the
 /// closure will become invalidated: all signal handlers connected to the closure will become
 /// disconnected, and any calls to [`Closure::invoke`] on the closure will be silently ignored.
 /// Internally, this is accomplished using [`Object::watch_closure`] on the watched object.
 ///
-/// The `@weak-allow-none` and `@strong` captures are also supported and behave the same as in
-/// [`clone!`](crate::clone!), as is aliasing captures with the `as` keyword. Notably, these
-/// captures are able to reference `Rc` and `Arc` values in addition to `Object` values.
+/// The `#[weak]`, `#[weak_allow_none]`, `#[strong]`, `#[to_owned]` captures are also supported and
+/// behave the same as in [`clone!`](crate::clone!), as is aliasing captures via `rename_to`.
+/// Similarly, upgrade failure of weak references can be adjusted via `#[upgrade_or]`,
+/// `#[upgrade_or_else]`, `#[upgrade_or_default]` and `#[upgrade_or_panic]`.
+///
+/// Notably, these captures are able to reference `Rc` and `Arc` values in addition to `Object`
+/// values.
 ///
 /// [`Closure`]: ../glib/closure/struct.Closure.html
 /// [`Closure::new`]: ../glib/closure/struct.Closure.html#method.new
@@ -368,9 +434,12 @@ pub fn clone(item: TokenStream) -> TokenStream {
 ///
 /// let closure = {
 ///     let obj = glib::Object::new::<glib::Object>();
-///     let closure = closure_local!(@watch obj => move || {
-///         obj.type_().name()
-///     });
+///     let closure = closure_local!(
+///         #[watch] obj,
+///         move || {
+///             obj.type_().name()
+///         },
+///     );
 ///     assert_eq!(closure.invoke::<String>(&[]), "GObject");
 ///     closure
 /// };
@@ -378,7 +447,7 @@ pub fn clone(item: TokenStream) -> TokenStream {
 /// closure.invoke::<()>(&[]);
 /// ```
 ///
-/// `@watch` has special behavior when connected to a signal:
+/// `#[watch]` has special behavior when connected to a signal:
 ///
 /// ```
 /// use glib;
@@ -390,10 +459,15 @@ pub fn clone(item: TokenStream) -> TokenStream {
 ///     let other = glib::Object::new::<glib::Object>();
 ///     obj.connect_closure(
 ///         "notify", false,
-///         closure_local!(@watch other as b => move |a: glib::Object, pspec: glib::ParamSpec| {
-///             let value = a.property_value(pspec.name());
-///             b.set_property(pspec.name(), &value);
-///         }));
+///         closure_local!(
+///             #[watch(rename_to = b)]
+///             other,
+///             move |a: glib::Object, pspec: glib::ParamSpec| {
+///                 let value = a.property_value(pspec.name());
+///                 b.set_property(pspec.name(), &value);
+///             },
+///         ),
+///     );
 ///     // The signal handler will disconnect automatically at the end of this
 ///     // block when `other` is dropped.
 /// }
@@ -411,10 +485,17 @@ pub fn clone(item: TokenStream) -> TokenStream {
 ///     let a = Arc::new(String::from("Hello"));
 ///     let b = Arc::new(String::from("World"));
 ///     let c = "!";
-///     let closure = closure!(@strong a, @weak-allow-none b, @to-owned c => move || {
-///         // `a` is Arc<String>, `b` is Option<Arc<String>>, `c` is a `String`
-///         format!("{} {}{}", a, b.as_ref().map(|b| b.as_str()).unwrap_or_else(|| "Moon"), c)
-///     });
+///     let closure = closure!(
+///         #[strong] a,
+///         #[weak_allow_none]
+///         b,
+///         #[to_owned]
+///         c,
+///         move || {
+///             // `a` is Arc<String>, `b` is Option<Arc<String>>, `c` is a `String`
+///             format!("{} {}{}", a, b.as_ref().map(|b| b.as_str()).unwrap_or_else(|| "Moon"), c)
+///         },
+///     );
 ///     assert_eq!(closure.invoke::<String>(&[]), "Hello World!");
 ///     closure
 /// };
@@ -423,7 +504,18 @@ pub fn clone(item: TokenStream) -> TokenStream {
 /// ```
 #[proc_macro]
 pub fn closure(item: TokenStream) -> TokenStream {
-    closure::closure_inner(item, "new")
+    // Check if this is an old-style closure macro invocation.
+    // These always start with an '@' punctuation.
+    let Some(first) = item.clone().into_iter().next() else {
+        return syn::Error::new(Span::call_site(), "expected a closure")
+            .to_compile_error()
+            .into();
+    };
+
+    match first {
+        TokenTree::Punct(ref p) if p.to_string() == "@" => closure_old::closure_inner(item, "new"),
+        _ => closure::closure_inner(item, "new"),
+    }
 }
 
 /// The same as [`closure!`](crate::closure!) but uses [`Closure::new_local`] as a constructor.
@@ -433,11 +525,24 @@ pub fn closure(item: TokenStream) -> TokenStream {
 /// [`Closure::new_local`]: ../glib/closure/struct.Closure.html#method.new_local
 #[proc_macro]
 pub fn closure_local(item: TokenStream) -> TokenStream {
-    closure::closure_inner(item, "new_local")
+    // Check if this is an old-style closure macro invocation.
+    // These always start with an '@' punctuation.
+    let Some(first) = item.clone().into_iter().next() else {
+        return syn::Error::new(Span::call_site(), "expected a closure")
+            .to_compile_error()
+            .into();
+    };
+
+    match first {
+        TokenTree::Punct(ref p) if p.to_string() == "@" => {
+            closure_old::closure_inner(item, "new_local")
+        }
+        _ => closure::closure_inner(item, "new_local"),
+    }
 }
 
-/// Derive macro for register a Rust enum in the GLib type system and derive the
-/// the [`glib::Value`] traits.
+/// Derive macro to register a Rust enum in the GLib type system and derive the
+/// [`glib::Value`] traits.
 ///
 /// # Example
 ///
@@ -458,6 +563,7 @@ pub fn closure_local(item: TokenStream) -> TokenStream {
 ///
 /// An enum can be registered as a dynamic type by setting the derive macro
 /// helper attribute `enum_dynamic`:
+///
 /// ```ignore
 /// use glib::prelude::*;
 /// use glib::subclass::prelude::*;
@@ -471,7 +577,7 @@ pub fn closure_local(item: TokenStream) -> TokenStream {
 /// ```
 ///
 /// As a dynamic type, an enum must be explicitly registered when the system
-/// loads the implementation (see [`TypePlugin`] and [`TypeModule`].
+/// loads the implementation (see [`TypePlugin`] and [`TypeModule`]).
 /// Therefore, whereas an enum can be registered only once as a static type,
 /// it can be registered several times as a dynamic type.
 ///
@@ -485,9 +591,10 @@ pub fn closure_local(item: TokenStream) -> TokenStream {
 /// when registering an enum as a dynamic type:
 ///
 /// - lazy registration: by default an enum is registered as a dynamic type
-/// when the system loads the implementation (e.g. when the module is loaded).
-/// Optionally setting `lazy_registration` to `true` postpones registration on
-/// the first use (when `static_type()` is called for the first time):
+///   when the system loads the implementation (e.g. when the module is loaded).
+///   Optionally setting `lazy_registration` to `true` postpones registration on
+///   the first use (when `static_type()` is called for the first time):
+///
 /// ```ignore
 /// #[derive(Debug, Copy, Clone, PartialEq, Eq, glib::Enum)]
 /// #[enum_type(name = "MyEnum")]
@@ -498,8 +605,9 @@ pub fn closure_local(item: TokenStream) -> TokenStream {
 /// ```
 ///
 /// - registration within [`TypeModule`] subclass or within [`TypePlugin`]
-/// subclass: an enum is usually registered as a dynamic type within a
-/// [`TypeModule`] subclass:
+///   subclass: an enum is usually registered as a dynamic type within a
+///   [`TypeModule`] subclass:
+///
 /// ```ignore
 /// #[derive(Debug, Copy, Clone, PartialEq, Eq, glib::Enum)]
 /// #[enum_type(name = "MyModuleEnum")]
@@ -524,6 +632,7 @@ pub fn closure_local(item: TokenStream) -> TokenStream {
 ///
 /// Optionally setting `plugin_type` allows to register an enum as a dynamic
 /// type within a [`TypePlugin`] subclass that is not a [`TypeModule`]:
+///
 /// ```ignore
 /// #[derive(Debug, Copy, Clone, PartialEq, Eq, glib::Enum)]
 /// #[enum_type(name = "MyPluginEnum")]
@@ -566,7 +675,7 @@ pub fn enum_derive(input: TokenStream) -> TokenStream {
 /// Default name is the flag identifier in CamelCase and default nick
 /// is the identifier in kebab-case.
 /// Combined flags should not be registered with the `GType` system
-/// and so needs to be tagged with the `#[flags_value(skip)]` attribute.
+/// and so need to be tagged with the `#[flags_value(skip)]` attribute.
 ///
 /// # Example
 ///
@@ -600,7 +709,7 @@ pub fn enum_derive(input: TokenStream) -> TokenStream {
 /// ```
 ///
 /// As a dynamic type, the flags must be explicitly registered when the system
-/// loads the implementation (see [`TypePlugin`] and [`TypeModule`].
+/// loads the implementation (see [`TypePlugin`] and [`TypeModule`]).
 /// Therefore, whereas the flags can be registered only once as a static type,
 /// they can be registered several times as a dynamic type.
 ///
@@ -614,9 +723,10 @@ pub fn enum_derive(input: TokenStream) -> TokenStream {
 /// registering the flags as a dynamic type:
 ///
 /// - lazy registration: by default the flags are registered as a dynamic type
-/// when the system loads the implementation (e.g. when the module is loaded).
-/// Optionally setting `lazy_registration` to `true` postpones registration on
-/// the first use (when `static_type()` is called for the first time):
+///   when the system loads the implementation (e.g. when the module is loaded).
+///   Optionally setting `lazy_registration` to `true` postpones registration on
+///   the first use (when `static_type()` is called for the first time):
+///
 /// ```ignore
 /// #[glib::flags(name = "MyFlags")]
 /// #[flags_dynamic(lazy_registration = true)]
@@ -626,8 +736,9 @@ pub fn enum_derive(input: TokenStream) -> TokenStream {
 /// ```
 ///
 /// - registration within [`TypeModule`] subclass or within [`TypePlugin`]
-/// subclass: the flags are usually registered as a dynamic type within a
-/// [`TypeModule`] subclass:
+///   subclass: the flags are usually registered as a dynamic type within a
+///   [`TypeModule`] subclass:
+///
 /// ```ignore
 /// #[glib::flags(name = "MyModuleFlags")]
 /// #[flags_dynamic]
@@ -680,13 +791,25 @@ pub fn flags(attr: TokenStream, item: TokenStream) -> TokenStream {
     let mut name = NestedMetaItem::<syn::LitStr>::new("name")
         .required()
         .value_required();
+    let mut allow_name_conflict_attr =
+        NestedMetaItem::<syn::LitBool>::new("allow_name_conflict").value_optional();
 
-    if let Err(e) = parse_nested_meta_items_from_stream(attr.into(), &mut [&mut name]) {
+    if let Err(e) = parse_nested_meta_items_from_stream(
+        attr.into(),
+        &mut [&mut name, &mut allow_name_conflict_attr],
+    ) {
         return e.to_compile_error().into();
     }
 
+    let allow_name_conflict = allow_name_conflict_attr.found
+        || allow_name_conflict_attr
+            .value
+            .map(|b| b.value())
+            .unwrap_or(false);
+
     let attr_meta = AttrInput {
         enum_name: name.value.unwrap(),
+        allow_name_conflict,
     };
 
     syn::parse::<syn::ItemEnum>(item)
@@ -803,7 +926,7 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 /// type Interfaces = ();
 /// ```
 ///
-/// If no `new()` or `with_class()` method is provide, the macro adds a `new()`
+/// If no `new()` or `with_class()` method is provided, the macro adds a `new()`
 /// implementation calling `Default::default()`. So the type needs to implement
 /// `Default`, or this should be overridden.
 ///
@@ -815,6 +938,7 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 ///
 /// An object subclass can be registered as a dynamic type by setting the macro
 /// helper attribute `object_class_dynamic`:
+///
 /// ```ignore
 /// #[derive(Default)]
 /// pub struct MyType;
@@ -825,7 +949,7 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 /// ```
 ///
 /// As a dynamic type, an object subclass must be explicitly registered when
-/// the system loads the implementation (see [`TypePlugin`] and [`TypeModule`].
+/// the system loads the implementation (see [`TypePlugin`] and [`TypeModule`]).
 /// Therefore, whereas an object subclass can be registered only once as a
 /// static type, it can be registered several times as a dynamic type.
 ///
@@ -839,10 +963,11 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 /// when registering an object subclass as a dynamic type:
 ///
 /// - lazy registration: by default an object subclass is registered as a
-/// dynamic type when the system loads the implementation (e.g. when the module
-/// is loaded). Optionally setting `lazy_registration` to `true` postpones
-/// registration on the first use (when `static_type()` is called for the first
-/// time):
+///   dynamic type when the system loads the implementation (e.g. when the module
+///   is loaded). Optionally setting `lazy_registration` to `true` postpones
+///   registration on the first use (when `static_type()` is called for the first
+///   time):
+///
 /// ```ignore
 /// #[derive(Default)]
 /// pub struct MyType;
@@ -853,8 +978,9 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 /// ```
 ///
 /// - registration within [`TypeModule`] subclass or within [`TypePlugin`]
-/// subclass: an object subclass is usually registered as a dynamic type within
-/// a [`TypeModule`] subclass:
+///   subclass: an object subclass is usually registered as a dynamic type within
+///   a [`TypeModule`] subclass:
+///
 /// ```ignore
 /// #[derive(Default)]
 /// pub struct MyModuleType;
@@ -879,6 +1005,7 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 ///
 /// Optionally setting `plugin_type` allows to register an object subclass as a
 /// dynamic type within a [`TypePlugin`] subclass that is not a [`TypeModule`]:
+///
 /// ```ignore
 /// #[derive(Default)]
 /// pub struct MyPluginType;
@@ -906,16 +1033,8 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 /// [`TypePluginExt::unuse`]: ../glib/gobject/type_plugin/trait.TypePluginExt.html#method.unuse
 #[proc_macro_attribute]
 pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
-    syn::parse::<syn::ItemImpl>(item)
-        .map_err(|_| {
-            syn::Error::new(
-                Span::call_site(),
-                object_subclass_attribute::WRONG_PLACE_MSG,
-            )
-        })
-        .and_then(|mut input| object_subclass_attribute::impl_object_subclass(&mut input))
-        .unwrap_or_else(syn::Error::into_compile_error)
-        .into()
+    let input = parse_macro_input!(item with object_impl_attributes::Input::parse_subclass);
+    object_impl_attributes::subclass::impl_object_subclass(input).into()
 }
 
 /// Macro for boilerplate of [`ObjectInterface`] implementations.
@@ -923,10 +1042,10 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 /// This adds implementations for the `get_type()` method, which should probably never be defined
 /// differently.
 ///
-/// It provides default values for the `Prerequisites` type parameter. If this present, the macro
+/// It provides default values for the `Prerequisites` type parameter. If this is present, the macro
 /// will use the provided value instead of the default.
 ///
-/// `Prerequisites` is interfaces for types that require a specific base class or interfaces.
+/// `Prerequisites` are interfaces for types that require a specific base class or interfaces.
 ///
 /// ```ignore
 /// type Prerequisites = ();
@@ -944,7 +1063,7 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 /// ```
 ///
 /// As a dynamic type, an object interface must be explicitly registered when
-/// the system loads the implementation (see [`TypePlugin`] and [`TypeModule`].
+/// the system loads the implementation (see [`TypePlugin`] and [`TypeModule`]).
 /// Therefore, whereas an object interface can be registered only once as a
 /// static type, it can be registered several times as a dynamic type.
 ///
@@ -958,9 +1077,10 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 /// behaviors when registering an object interface as a dynamic type:
 ///
 /// - lazy registration: by default an object interface is registered as a
-/// dynamic type when the system loads the implementation (e.g. when the module
-/// is loaded). Optionally setting `lazy_registration` to `true` postpones
-/// registration on the first use (when `type_()` is called for the first time):
+///   dynamic type when the system loads the implementation (e.g. when the module
+///   is loaded). Optionally setting `lazy_registration` to `true` postpones
+///   registration on the first use (when `type_()` is called for the first time):
+///
 /// ```ignore
 /// pub struct MyInterface {
 ///     parent: glib::gobject_ffi::GTypeInterface,
@@ -971,8 +1091,9 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 /// ```
 ///
 /// - registration within [`TypeModule`] subclass or within [`TypePlugin`]
-/// subclass: an object interface is usually registered as a dynamic type
-/// within a [`TypeModule`] subclass:
+///   subclass: an object interface is usually registered as a dynamic type
+///   within a [`TypeModule`] subclass:
+///
 /// ```ignore
 /// pub struct MyModuleInterface {
 ///     parent: glib::gobject_ffi::GTypeInterface,
@@ -997,6 +1118,7 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 ///
 /// Optionally setting `plugin_type` allows to register an object interface as
 /// a dynamic type within a [`TypePlugin`] subclass that is not a [`TypeModule`]:
+///
 /// ```ignore
 /// pub struct MyPluginInterface {
 ///     parent: glib::gobject_ffi::GTypeInterface,
@@ -1024,16 +1146,8 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 /// [`TypePluginExt::unuse`]: ../glib/gobject/type_plugin/trait.TypePluginExt.html#method.unuse///
 #[proc_macro_attribute]
 pub fn object_interface(_attr: TokenStream, item: TokenStream) -> TokenStream {
-    syn::parse::<syn::ItemImpl>(item)
-        .map_err(|_| {
-            syn::Error::new(
-                Span::call_site(),
-                object_interface_attribute::WRONG_PLACE_MSG,
-            )
-        })
-        .and_then(|mut input| object_interface_attribute::impl_object_interface(&mut input))
-        .unwrap_or_else(syn::Error::into_compile_error)
-        .into()
+    let input = parse_macro_input!(item with object_impl_attributes::Input::parse_interface);
+    object_impl_attributes::interface::impl_object_interface(input).into()
 }
 
 /// Macro for deriving implementations of [`glib::clone::Downgrade`] and
@@ -1059,7 +1173,13 @@ pub fn object_interface(_attr: TokenStream, item: TokenStream) -> TokenStream {
 ///
 /// let fancy_label = FancyLabel::new("Look at me!");
 /// let button = gtk::ButtonBuilder::new().label("Click me!").build();
-/// button.connect_clicked(clone!(@weak fancy_label => move || fancy_label.flip()));
+/// button.connect_clicked(
+///     clone!(
+///         #[weak]
+///         fancy_label,
+///         move || fancy_label.flip(),
+///     ),
+/// );
 /// ```
 ///
 /// ## Generic New Type
@@ -1142,16 +1262,16 @@ pub fn downgrade(input: TokenStream) -> TokenStream {
 /// enum variant, or just `s` if this is a C-style enum. Some additional attributes are supported
 /// for enums:
 /// - `#[variant_enum(repr)]` to serialize the enum variant as an integer type instead of `s`.  The
-/// `#[repr]` attribute must also be specified on the enum with a sized integer type, and the type
-/// must implement `Copy`.
+///   `#[repr]` attribute must also be specified on the enum with a sized integer type, and the type
+///   must implement `Copy`.
 /// - `#[variant_enum(enum)]` uses [`EnumClass`] to serialize/deserialize as nicks. Meant for use
-/// with [`glib::Enum`](Enum).
+///   with [`glib::Enum`](Enum).
 /// - `#[variant_enum(flags)]` uses [`FlagsClass`] to serialize/deserialize as nicks. Meant for use
-/// with [`glib::flags`](macro@flags).
+///   with [`glib::flags`](macro@flags).
 /// - `#[variant_enum(enum, repr)]` serializes as `i32`. Meant for use with [`glib::Enum`](Enum).
-/// The type must also implement `Copy`.
+///   The type must also implement `Copy`.
 /// - `#[variant_enum(flags, repr)]` serializes as `u32`. Meant for use with
-/// [`glib::flags`](macro@flags).
+///   [`glib::flags`](macro@flags).
 ///
 /// # Example
 ///
@@ -1236,8 +1356,8 @@ pub fn cstr_bytes(item: TokenStream) -> TokenStream {
 /// | --- | --- | --- | --- |
 /// | `name = "literal"` | The name of the property | field ident where `_` (leading and trailing `_` are trimmed) is replaced into `-` | `#[property(name = "prop-name")]` |
 /// | `type = expr` | The type of the property | inferred | `#[property(type = i32)]` |
-/// | `get [= expr]` | Specify that the property is readable and use `PropertyGet::get` [or optionally set a custom internal getter] | | `#[property(get)]`, `#[property(get = get_prop)]`, or `[property(get = \|_\| 2)]` |
-/// | `set [= expr]` | Specify that the property is writable and use `PropertySet::set` [or optionally set a custom internal setter] | | `#[property(set)]`, `#[property(set = set_prop)]`, or `[property(set = \|_, val\| {})]` |
+/// | `get [= expr]` | Specify that the property is readable and use [`PropertyGet::get`] [or optionally set a custom internal getter] | | `#[property(get)]`, `#[property(get = get_prop)]`, or `[property(get = \|_\| 2)]` |
+/// | `set [= expr]` | Specify that the property is writable and use [`PropertySet::set`] [or optionally set a custom internal setter] | | `#[property(set)]`, `#[property(set = set_prop)]`, or `[property(set = \|_, val\| {})]` |
 /// | `override_class = expr` | The type of class of which to override the property from | | `#[property(override_class = SomeClass)]` |
 /// | `override_interface = expr` | The type of interface of which to override the property from | | `#[property(override_interface = SomeInterface)]` |
 /// | `nullable` | Whether to use `Option<T>` in the generated setter method |  | `#[property(nullable)]` |
@@ -1273,32 +1393,32 @@ pub fn cstr_bytes(item: TokenStream) -> TokenStream {
 /// by assigning an expression to `get`/`set` `#[property]` attributes: `#[property(get = |_| 2, set)]` or `#[property(get, set = custom_setter_func)]`.
 ///
 /// # Supported types
-/// Every type implementing the trait `Property` is supported.
-/// The type `Option<T>` is supported as a property only if `Option<T>` implements `ToValueOptional`.
+/// Every type implementing the trait [`Property`] is supported.
+/// The type `Option<T>` is supported as a property only if `Option<T>` implements [`ToValueOptional`].
 /// Optional types also require the `nullable` attribute: without it, the generated setter on the wrapper type
 /// will take `T` instead of `Option<T>`, preventing the user from ever calling the setter with a `None` value.
 ///
 /// ## Adding support for custom types
-/// ### Types wrapping an existing `T: glib::value::ToValue + glib::HasParamSpec`
+/// ### Types wrapping an existing <code>T: [ToValue] + [HasParamSpec]</code>
 /// If you have declared a newtype as
 /// ```rust
 /// struct MyInt(i32);
 /// ```
-/// you can use it as a property by deriving `glib::ValueDelegate`.
+/// you can use it as a property by deriving [`ValueDelegate`].
 ///
 /// ### Types with inner mutability
-/// The trait `glib::Property` must be implemented.
-/// The traits `PropertyGet` and `PropertySet` should be implemented to enable the Properties macro
+/// The trait [`Property`] must be implemented.
+/// The traits [`PropertyGet`] and [`PropertySet`] should be implemented to enable the Properties macro
 /// to generate a default internal getter/setter.
-/// If possible, implementing `PropertySetNested` is preferred over `PropertySet`, because it
+/// If possible, implementing [`PropertySetNested`] is preferred over `PropertySet`, because it
 /// enables this macro to access the contained type and provide access to its fields,
 /// using the `member = $structfield` syntax.
 ///
-/// ### Types without `glib::HasParamSpec`
-/// If you have encountered a type `T: glib::value::ToValue`, inside the `gtk-rs` crate, which doesn't implement `HasParamSpec`,
+/// ### Types without [`HasParamSpec`][HasParamSpec]
+/// If you have encountered a type <code>T: [ToValue]</code>, inside the gtk-rs crate, which doesn't implement [`HasParamSpec`][HasParamSpec],
 /// then it's a bug and you should report it.
-/// If you need to support a `ToValue` type with a `ParamSpec` not provided by `gtk-rs`, then you need to
-/// implement `glib::HasParamSpec` on that type.
+/// If you need to support a `ToValue` type with a [`ParamSpec`] not provided by gtk-rs, then you need to
+/// implement `HasParamSpec` on that type.
 ///
 /// # Example
 /// ```
@@ -1365,6 +1485,18 @@ pub fn cstr_bytes(item: TokenStream) -> TokenStream {
 ///   assert_eq!(myfoo.fizz(), "custom set: test value".to_string());
 /// }
 /// ```
+///
+/// [`Property`]: ../glib/property/trait.Property.html
+/// [`PropertyGet`]: ../glib/property/trait.PropertyGet.html
+/// [`PropertyGet::get`]: ../glib/property/trait.PropertyGet.html#tymethod.get
+/// [`PropertySet`]: ../glib/property/trait.PropertySet.html
+/// [`PropertySet::set`]: ../glib/property/trait.PropertySet.html#tymethod.set
+/// [`PropertySetNested`]: ../glib/property/trait.PropertySetNested.html
+/// [`ObjectExt::property`]: ../glib/object/trait.ObjectExt.html#tymethod.property
+/// [HasParamSpec]: ../glib/trait.HasParamSpec.html
+/// [`ParamSpec`]: ../glib/struct.ParamSpec.html
+/// [`ToValueOptional`]: ../glib/value/trait.ToValueOptional.html
+/// [ToValue]: ../glib/value/trait.ToValue.html
 #[allow(clippy::needless_doctest_main)]
 #[proc_macro_derive(Properties, attributes(properties, property))]
 pub fn derive_props(input: TokenStream) -> TokenStream {
